package communicationservices;
import dataservices.*;

/**
 * <p>Title: </p>
 * <p>Description: POMESClient will be created by a Protocol manager and do lookups
 * into a session table for device ip, will send a command, wait for the proper
 * response (i.e. it may recieve duplicate get commands if POMES response isn't fast
 * enough), send more commands
 * and so on based on what it gets from a DMC (or calling class)</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author unascribed
 * @version 0.3
 */

import java.io.*;
import java.net.*;

public class POMESClient
       implements SNMPProxyConstants
{
  private static String pomesIP        = POMES_IP;//"127.0.0.1";
  private static int pomesPort         = POMES_PORT;
  private static String name           = NAME;
  private static String password       = PASSWORD;

  private Socket XMLSocket             = null;
  private DataOutputStream out         = null;
  private BufferedReader in            = null;

  private boolean sessionOpen          = false;

  public POMESClient(SNMPProxyConfig config)
  {
    pomesIP = config.getPomesIP();
    pomesPort = config.getPomesPort();
    name = config.getName();
    password = config.getPassword();

  }

  public POMESClient(String ip, int port)
  {
    pomesIP = ip;
    pomesPort = port;
  }

  public boolean openPOMESSession()
  {
    try
    { XMLSocket = new Socket(pomesIP, pomesPort);
      out       = new DataOutputStream(XMLSocket.getOutputStream());
      in        = new BufferedReader(
	          new InputStreamReader(XMLSocket.getInputStream()));
    }
    catch (UnknownHostException e)
    { System.err.println("Could not find host at " + pomesIP);
      System.exit(1);
    }
    catch (IOException e)
    { System.err.println("Couldn't get I/O for connection to: "
                               + pomesIP + " on port " + pomesPort);
      System.exit(1);
    }
    this.receiveResponse();
    sessionOpen = true;
    return true;
  }

  public void sendCommand(String data)
  {
    try
    {
        out.writeBytes(data);
        System.out.println("CLIENT SENT:\n" + data);
    }
    catch (IOException e)
    {
        System.out.println("IOException: " + e);
    }
  }
  public String receiveResponse()
  {
    String line = "";
    String temp = "";
    try
    { while ((line = in.readLine()) != null)
      { temp += line + "\n";
        if (line.indexOf("</POM-API>") != -1)
        {
          return temp;
        }
      }
      in.skip(1);
      temp += "\n";
    }
    catch (IOException e)
    { System.out.println("IOException: " + e);
    }
    System.out.println("CLIENT RECEIVED:\n" + temp);
    return temp;
  }

  public void closePOMESSession()
  {
    try
    {
      sessionOpen = false;
      out.close();
      in.close();
      XMLSocket.close();
    }
    catch (IOException e)
    {
      System.out.println("IOException: " + e);
    }
  }

  public boolean POMESSessionOpen()
  {
    return sessionOpen;
  }
}