package controlservices;

import communicationservices.*;
import dataservices.*;

import java.net.DatagramPacket;

/**
 * <p>Title: ProtocolManager</p>
 * <p>Description: This class uses the UDPComManager to listen for incoming
 * udp messages and create protocol handlers to deal with them.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Printronix</p>
 * @author Christopher M. King (kingc@uci.edu)
 * @version 0.5
 */

public class ProxyManager
{

  UDPComManager udpComManager;
  SNMPSecurityManager securityManager;
  Log proxyLog;
  SNMPSessionManager sessionManager;
  SNMPProxyConfig config;

  public ProxyManager()
  {
     System.out.println("Starting SNMP Proxy...");
     config = new SNMPProxyConfig();

     udpComManager = new UDPComManager(this);
     sessionManager = new SNMPSessionManager(config);
     proxyLog = new Log("proxy.log");

     securityManager = new SNMPSecurityManager(proxyLog);
     securityManager.setCommunityName(config.getCommunity());
  }

  /**
   * Begins listening for incoming udp messages
   */
  public void start()
  {
    udpComManager.listen(161);
  }

  /**
   * Helper fucntion that handles incoming UDPPackets, normally called internally
   * when the a UDPPacket is recieved by the ProxyManager that creates a new
   * protocol thread to deal with it.
   *
   * @param udpPacket A UDPPacket to be dealt with
   */
  public void recievedUDP(DatagramPacket udpPacket)
  {
      ProtocolHandler p = new ProtocolHandler(udpPacket, sessionManager, securityManager, udpComManager, proxyLog, config);
      p.start();
  }
}
